import { Injectable } from '@nestjs/common';

import { InjectDataSource, InjectRepository } from '@nestjs/typeorm';
import { CONNECTION_NAME } from 'src/core/strings/constants/base.constants';
import { Between, DataSource, In, Repository } from 'typeorm';
import {
  QueueItemModel,
  QueueModel,
  QueueOrderModel,
} from '../models/queue.model';
import { BaseReadService } from 'src/core/modules/data/service/base-read.service';
import { BaseDataService } from 'src/core/modules/data/service/base-data.service';
import * as moment from 'moment';
import { ItemQueueModel } from 'src/modules/item-related/item-queue/data/models/item-queue.model';
import * as math from 'mathjs';

@Injectable()
export class QueueDataService extends BaseReadService<QueueModel> {
  constructor(
    @InjectRepository(QueueModel, CONNECTION_NAME.DEFAULT)
    private repo: Repository<QueueModel>,

    @InjectRepository(ItemQueueModel, CONNECTION_NAME.DEFAULT)
    private itemQueueRepo: Repository<ItemQueueModel>,
  ) {
    super(repo);
  }

  async queueTimes(item_queue_id: string): Promise<any> {
    const queueTimes = {};
    let now = moment().valueOf();
    const itemQueue = await this.itemQueueRepo.findOne({
      relations: ['items'],
      where: {
        id: item_queue_id,
      },
    });

    const times = itemQueue.items.map((item) => item.play_estimation ?? 0);
    const average = math.mean(times) * 60 * 1000; // change average minute to milliseconds
    const queues = await this.repo.find({
      where: {
        item_queue_id,
        status: 'waiting',
      },
    });

    queueTimes[queues[0].id] = now; // first queue will be now

    for (let i = 1; i < queues.length; i++) {
      const queue = queues[i];
      // duration will be total qty multiple by average
      const duration = queue.qty * average;

      // time to call will be now + duration
      const time = now + duration;
      queueTimes[queue.id] = time;

      // update now to last call time
      now = time;
    }

    return queueTimes;
  }
}

@Injectable()
export class QueueService extends BaseDataService<QueueModel> {
  constructor(
    @InjectRepository(QueueModel, CONNECTION_NAME.DEFAULT)
    private repo: Repository<QueueModel>,

    @InjectRepository(QueueItemModel, CONNECTION_NAME.DEFAULT)
    private item: Repository<QueueItemModel>,

    @InjectDataSource(CONNECTION_NAME.DEFAULT)
    private dataSource: DataSource,
  ) {
    super(repo);
  }

  async queues(ids: string[]) {
    const start = moment().startOf('day').valueOf();
    const end = moment().endOf('day').valueOf();
    return this.repo.find({
      where: {
        item_queue_id: In(ids),
        time: Between(start, end),
      },
      order: {
        time: 'ASC',
      },
    });
  }

  async getTicketItems(ticket_id: string, item_id: string) {
    return this.item.findOneOrFail({
      relations: ['item'],
      where: [
        {
          ticket_id,
          item_id,
        },
        {
          ticket_id,
          item: { item_queue_id: item_id },
        },
      ],
    });
  }

  async updateItemQty(item_id: string, qty: number): Promise<void> {
    const query = `UPDATE queue_items SET qty = qty - ${qty} WHERE id = '${item_id}'`;
    this.dataSource.query(query);
  }
}

@Injectable()
export class QueueOrderService extends BaseDataService<QueueOrderModel> {
  constructor(
    @InjectRepository(QueueOrderModel, CONNECTION_NAME.DEFAULT)
    private repo: Repository<QueueOrderModel>,
  ) {
    super(repo);
  }
}
